/********************************************************************
*                                                                   *
* Laboratoire MASI CAO-VLSI, UPMC, Copyright 1991 1992 1993 1994    *
*                                                                   *
* Software support Email : cao-vlsi@masi.ibp.fr                     *
*                                                                   *
* Authors : Zouina AKTOUF &                                         *
*           El Arabi RHOMARI &                                      *
*           Jose MARTINS DOS SANTOS                                 *
*                                                                   *
* Supervision & Modifications : Lotfi BEN AMMAR                     *
*                                                                   *
********************************************************************/
extern char *checkname();
extern void affcon();
extern chain_list *existsig();
extern chain_list *addpos();
extern ConnectorList *insertcon_inchan();
extern void Warning();
extern void Error();
extern long strnbr();
extern char *nbrstr();
extern long strrch();
extern long delta_x();
extern long delta_y();
extern char orient_r();
extern char orient_l();
extern char prient_rl();
extern char Orient();
extern long deltab_y();
extern long deltab_x();
extern chain_list *addchannel();
extern chain_list *addpos();
extern chain_list *addnwell();
extern chain_list *addseg();
extern chain_list *addvia();
extern TRACK *addtrack();
extern char manorme();
